<?php

class Sipgate
{
    const MESSAGE_TYPE_TEXT         = 't';
    const MESSAGE_TYPE_LONG_TEXT    = 'c';
    const MESSAGE_TYPE_RICH         = 'utf8';

    const API_ENDPOINT = 'http://gate1.goyyamobile.com/sms/sendsms.asp';

    /**
     * @var bool
     */
    public static $test_mode = false;

    /**
     * @var resource
     */
    protected $_curl;

    /**
     * @var array
     */
    protected $_curlOptions  = [];

    /**
     * @var string
     */
    protected $_id;

    /**
     * @var string
     */
    protected $_password;

    /**
     * @var string
     */
    protected $_authToken;

    /**
     * @var string
     */
    protected $_sender;

    /**
     * Accepts user id with password or auth token.
     *
     * @param   string          $idOrToken
     * @param   string|null     $password
     * @throws  Exception
     */
    public function __construct($idOrToken, $password = null)
    {
        if (!empty($password))
        {
            $this->setUserCredentials($idOrToken, $password);
        }
        else if (!empty($idOrToken))
        {
            $this->_authToken = $idOrToken;
        }
        else
        {
            throw new Exception('Incorrect usage');
        }

        $this->_curl = curl_init();
        $this->_setCurlOption(CURLOPT_RETURNTRANSFER, true);
    }

    /**
     * @param   string          $option     CURLOPT option constant
     * @param   mixed           $value
     * @return  Sipgate
     */
    protected function _setCurlOption($option, $value)
    {
        $this->_curlOptions[$option] = $value;
        return $this;
    }

    /**
     * @param   string          $id
     * @param   string          $password
     * @return  Sipgate
     */
    public function setUserCredentials($id, $password)
    {
        $this->_id = $id;
        $this->_password = $password;
        $this->_authToken = null;

        return $this;
    }

    /**
     * @param   string          $authToken
     * @return  Sipgate
     */
    public function setAuthToken($authToken)
    {
        $this->_id = null;
        $this->_password = null;
        $this->_authToken = $authToken;

        return $this;
    }

    /**
     * @param   string          $sender
     * @return  Sipgate
     */
    public function setSender($sender)
    {
        $this->_sender = $sender;

        return $this;
    }

    /**
     * @param   array           $paramsArray
     */
    protected function _addCredentials(&$paramsArray)
    {
        if (isset($this->_id) && isset($this->_password))
        {
            $paramsArray['id'] = $this->_id;
            $paramsArray['password'] = $this->_password;
        }
    }

    /**
     * @return  array
     */
    protected function _makeRequest()
    {
        curl_setopt_array($this->_curl, $this->_curlOptions);
        $response = curl_exec($this->_curl);

        return array(
            'code'  => curl_getinfo($this->_curl, CURLINFO_HTTP_CODE),
            'body'  => $response
        );
    }

    /**
     * @param   string          $receiver
     * @param   string          $message
     * @param   string          $type           See Sipgate::MESSAGE_TYPE_*
     * @return  mixed
     * @throws  Exception
     */
    public function sendMessage($receiver, $message, $type = Sipgate::MESSAGE_TYPE_TEXT)
    {
        $params = [
            'sender'    => $this->_sender,
            'receiver'  => $receiver,
            'msg'       => $message,
            'msgtype'   => $type,
            'getId'     => 1
        ];

        if (self::$test_mode)
        {
            $params['test'] = 1;
        }

        $this->_addCredentials($params);

        $this->_setCurlOption(CURLOPT_HTTPGET, true)
            ->_setCurlOption(CURLOPT_CUSTOMREQUEST, 'GET')
            ->_setCurlOption(CURLOPT_URL, self::API_ENDPOINT . '?' . http_build_query($params));

        $response = $this->_makeRequest();

        if (!isset($response['code']) || $response['code'] != 200)
        {
            throw new Exception('Request failed, status code ' . (isset($response['code']) ? $response['code'] : null));
        }

        if (strpos($response['body'], 'OK') === false)
        {
            throw new Exception('Unexpected response "' . $response['body'] . '"');
        }

        preg_match('/OK \((.*)\)/', $response['body'], $matches);
        $messageId = isset($matches[1]) ? $matches[1] : null;

        return $messageId ?: $response['body'];
    }

    /**
     * @return  int
     * @throws  Exception
     */
    public function getLimit()
    {
        $params = ['getLimit' => 1];

        $this->_addCredentials($params);

        $this->_setCurlOption(CURLOPT_HTTPGET, true)
            ->_setCurlOption(CURLOPT_CUSTOMREQUEST, 'GET')
            ->_setCurlOption(CURLOPT_URL, self::API_ENDPOINT . '?' . http_build_query($params));

        $response = $this->_makeRequest();

        if (!isset($response['code']) || $response['code'] != 200)
        {
            throw new Exception('Request failed, status code ' . (isset($response['code']) ? $response['code'] : null));
        }

        return (int) $response['body'];
    }
}
